@@ -2,7 +2,7 @@
 #
 # Copyright (c) 2004-2006 TINY SPRL. (http://tiny.be) All Rights Reserved.
 #
-# $Id: form.py 5608 2007-02-09 08:59:08Z ced $
+# $Id$
 #
 # WARNING: This program as such is intended to be used by professional
 # programmers who take the whole responsability of assessing all potential
@@ -52,13 +52,20 @@ import gc
 from observator import oregistry
 from widget.screen import Screen
 
+
 class form(object):
-	def __init__(self, model, res_id=False, domain=[], view_type=None, view_ids=[], window=None, context={}):
+
+	def __init__(self, model, res_id=False, domain=None, view_type=None,
+			view_ids=None, window=None, context=None, name=False, limit=80,
+			auto_refresh=False):
 		if not view_type:
 			view_type = ['form','tree']
-		else:
-			if view_type[0]=='tree' and not res_id:
-				res_id = rpc.session.rpc_exec_auth('/object', 'execute', model, 'search', domain)
+		if domain is None:
+			domain = []
+		if view_ids is None:
+			view_ids = []
+		if context is None:
+			context = {}
 
 		fields = {}
 		self.model = model
@@ -71,59 +78,76 @@ class form(object):
 		self.domain = domain
 		self.context = context
 
-		self.screen = Screen(self.model, view_type=view_type, context=self.context, view_ids=view_ids, domain=domain, hastoolbar=options.options['form.toolbar'])
+		self.screen = Screen(self.model, view_type=view_type,
+				context=self.context, view_ids=view_ids, domain=domain,
+				hastoolbar=options.options['form.toolbar'], show_search=True,
+				window=self.window, limit=limit, readonly=bool(auto_refresh))
 		self.screen.signal_connect(self, 'record-message', self._record_message)
+		self.screen.widget.show()
 		oregistry.add_receiver('misc-message', self._misc_message)
 
-		self.name = self.screen.current_view.title
+		if not name:
+			self.name = self.screen.current_view.title
+		else:
+			self.name = name
+		vp = gtk.Viewport()
+		vp.set_shadow_type(gtk.SHADOW_NONE)
+		vp.add(self.screen.widget)
+		vp.show()
 		self.sw = gtk.ScrolledWindow()
 		self.sw.set_shadow_type(gtk.SHADOW_NONE)
 		self.sw.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
-		self.sw.add_with_viewport(self.screen.widget)
-		vp = self.sw.get_child()
-		vp.set_shadow_type(gtk.SHADOW_NONE)
-		self.sw.show_all()
+		self.sw.add(vp)
+		self.sw.show()
 		
 		self.has_backup = False
 		self.backup = {}
 
 		self.widget.pack_start(self.sw)
 		self.handlers = {
-			'but_new': ('New',self.sig_new),
-			'but_copy': ('Copy',self.sig_copy),
-			'but_save': ('Save',self.sig_save),
-			'but_save_as': ('Save As',self.sig_save_as),
-			'but_import': ('Import',self.sig_import),
-			'but_print_repeat': ('Repeat',self.sig_print_repeat),
-			'but_remove': ('Remove',self.sig_remove),
-			'but_search': ('Search',self.sig_search),
-			'but_previous': ('Previous',self.sig_previous),
-			'but_next': ('Next', self.sig_next),
-			#'but_preference': ('Preferences', self.sig_preference),
-			'but_goto_id': ('Go To ID...', self.sig_goto),
-			'but_log': ('Access log', self.sig_logs),
-			'but_print': ('Print', self.sig_print),
-			'but_reload': (_('Reload'),self.sig_reload),
-			'but_print_html': ('Print', self.sig_print_html),
-			'but_action': ('Print', self.sig_action),
-			'but_switch': ('Switch', self.sig_switch),
-			'but_attach': ('Attach', self.sig_attach),
-			'but_close': ('Close',self.sig_close)
+			'but_new': self.sig_new,
+			'but_copy': self.sig_copy,
+			'but_save': self.sig_save,
+			'but_save_as': self.sig_save_as,
+			'but_import': self.sig_import,
+			'but_print_repeat': self.sig_print_repeat,
+			'but_remove': self.sig_remove,
+			'but_search': self.sig_search,
+			'but_previous': self.sig_previous,
+			'but_next': self.sig_next,
+			#'but_preference': self.sig_preference,
+			'but_goto_id': self.sig_goto,
+			'but_log': self.sig_logs,
+			'but_print': self.sig_print,
+			'but_reload': self.sig_reload,
+			'but_print_html': self.sig_print_html,
+			'but_action': self.sig_action,
+			'but_switch': self.sig_switch,
+			'but_attach': self.sig_attach,
+			'but_close': self.sig_close,
 		}
 		if res_id:
 			if isinstance(res_id, int):
 				res_id = [res_id]
 			self.screen.load(res_id)
 		else:
-			if len(view_type) and view_type[0]=='form':
+			if self.screen.current_view.view_type == 'form':
 				self.sig_new(autosave=False)
+			if self.screen.current_view.view_type in ('tree', 'graph', 'calendar'):
+				self.screen.search_filter()
+
+		if auto_refresh and int(auto_refresh):
+			gobject.timeout_add(int(auto_refresh) * 1000, self.sig_reload)
 
 	def sig_goto(self, *args):
 		if not self.modified_save():
 			return
 		glade2 = glade.XML(common.terp_path("terp.glade"),'dia_goto_id',gettext.textdomain())
-		win = glade2.get_widget('dia_goto_id')
 		widget = glade2.get_widget('goto_spinbutton')
+		win = glade2.get_widget('dia_goto_id')
+		win.set_transient_for(self.window)
+		win.show_all()
+
 		response = win.run()
 		win.destroy()
 		if response == gtk.RESPONSE_OK:
@@ -179,12 +203,6 @@ class form(object):
 		return True
 
 	def sig_switch(self, widget=None, mode=None):
-		#
-		# Verifier pourquoi:
-		#	1. Modif
-		#	2. Switch & Cancel
-		#	3. Re-Switch -> Question car modified
-		#
 		if not self.modified_save():
 			return
 		self.screen.switch_view()
@@ -218,20 +236,28 @@ class form(object):
 		return True
 
 	def sig_remove(self, widget=None):
-		if common.sur(_('Are you sure to remove this record ?')):
+		if self.screen.current_view.view_type == 'form':
+			msg = _('Are you sure to remove this record ?')
+		else:
+			msg = _('Are you sure to remove those records ?')
+		if common.sur(msg):
 			id = self.screen.remove(unlink=True)
 			if not id:
-				self.message_state(_('Resource not removed !'))
+				self.message_state(_('Resources not removed !'))
 			else:
-				self.message_state(_('Resource removed.'))
+				self.message_state(_('Resources removed.'))
 
 	def sig_import(self, widget=None):
 		fields = []
+		while(self.screen.view_to_load):
+			self.screen.load_view_to_load()
 		win = win_import.win_import(self.model, self.screen.fields, fields, parent=self.window)
 		res = win.go()
 
 	def sig_save_as(self, widget=None):
 		fields = []
+		while(self.screen.view_to_load):
+			self.screen.load_view_to_load()
 		win = win_export.win_export(self.model, self.screen.ids_get(), self.screen.fields, fields, parent=self.window)
 		res = win.go()
 
@@ -240,25 +266,26 @@ class form(object):
 			if not self.modified_save():
 				return
 		self.screen.new()
+		self.message_state('')
 	
 	def sig_copy(self, *args):
 		if not self.modified_save():
 			return
 		res_id = self._id_get()
-		new_id = rpc.session.rpc_exec_auth('/object', 'execute', self.model, 'copy', res_id, {}, rpc.session.context)
-		self.screen.load([new_id])
-		self.message_state(_('Working now on the duplicated document !'))
+		ctx = self.context.copy()
+		ctx.update(rpc.session.context)
+		new_id = rpc.session.rpc_exec_auth('/object', 'execute', self.model, 'copy', res_id, {}, ctx)
+		if new_id:
+			self.screen.load([new_id])
+			self.message_state(_('Working now on the duplicated document !'))
 
 	def _form_save(self, auto_continue=True):
 		pass
 
 	def sig_save(self, widget=None, sig_new=True, auto_continue=True):
-		modification = self.screen.current_model.id
 		id = self.screen.save_current()
 		if id:
 			self.message_state(_('Document saved !'))
-			if not modification:
-				self.screen.new()
 		else:
 			self.message_state(_('Invalid form !'))
 		return bool(id)
@@ -276,20 +303,21 @@ class form(object):
 		self.message_state('')
 
 	def sig_reload(self):
+		if not hasattr(self, 'screen'):
+			return False
 		if self.screen.current_view.view_type == 'form':
 			self.screen.cancel_current()
 			self.screen.display()
 		else:
 			id = self.screen.id_get()
-			ids = self.screen.ids_get()
-			self.screen.clear()
-			self.screen.load(ids)
+			self.screen.search_filter()
 			for model in self.screen.models:
 				if model.id == id:
 					self.screen.current_model = model
 					self.screen.display()
 					break
 		self.message_state('')
+		return True
 
 	def sig_action(self, keyword='client_action_multi', previous=False, report_type='pdf', adds={}):
 		ids = self.screen.ids_get()
@@ -302,6 +330,10 @@ class form(object):
 			if not id:
 				return False
 			ids = [id]
+		if self.screen.current_view.view_type == 'tree':
+			sel_ids = self.screen.current_view.sel_ids_get()
+			if sel_ids:
+				ids = sel_ids
 		if len(ids):
 			obj = service.LocalService('action.main')
 			if previous and self.previous_action:
@@ -321,7 +353,7 @@ class form(object):
 		self.sig_action('client_print_multi', report_type='html')
 
 	def sig_print(self):
-		self.sig_action('client_print_multi', adds={'Print Screen': {'report_name':'printscreen.list', 'name':'Print Screen', 'type':'ir.actions.report.xml'}})
+		self.sig_action('client_print_multi', adds={_('Print Screen'): {'report_name':'printscreen.list', 'name':_('Print Screen'), 'type':'ir.actions.report.xml'}})
 
 	def sig_search(self, widget=None):
 		if not self.modified_save():
@@ -367,17 +399,18 @@ class form(object):
 		else:
 			self.message_state(_('No resource selected !'))
 
-	def modified_save(self):
+	def modified_save(self, reload=True):
 		if self.screen.is_modified():
 			value = common.sur_3b(_('This record has been modified\ndo you want to save it ?'))
 			if value == 'ok':
 				return self.sig_save()
 			elif value == 'ko':
-				self.sig_reload()
+				if reload:
+					self.sig_reload()
 				return True
 			else:
 				return False
 		return True
 
 	def sig_close(self, urgent=False):
-		return self.modified_save()
+		return self.modified_save(reload=False)
