@@ -1,7 +1,7 @@
 ##############################################################################
 #
 # Copyright (c) 2004 TINY SPRL. (http://tiny.be) All Rights Reserved.
-#						Fabien Pinckaers <fp@tiny.Be>
+#                        Fabien Pinckaers <fp@tiny.Be>
 #
 # WARNING: This program as such is intended to be used by professional
 # programmers who take the whole responsability of assessing all potential
@@ -44,18 +44,22 @@ import tempfile
 
 class win_attach(object):
 	def __init__(self, model, id, parent=None):
-		self.glade = glade.XML(common.terp_path("terp.glade"),'win_attach',gettext.textdomain())
+		self.glade = glade.XML(common.terp_path("terp.glade"), 'win_attach',
+				gettext.textdomain())
 		self.win = self.glade.get_widget('win_attach')
+		self.win.set_icon(common.TINYERP_ICON)
 		if parent:
 			self.win.set_transient_for(parent)
+		self.parent = parent
 		self.ressource = (model, id)
 
 		self.view = gtk.TreeView()
 		hb = self.glade.get_widget('vp_attach')
 		hb.add(self.view)
 
-		self.view.get_selection().set_mode(gtk.SELECTION_SINGLE)
-		self.view.connect('row_activated', self._sig_row_activated)
+		selection = self.view.get_selection()
+		selection.set_mode(gtk.SELECTION_SINGLE)
+		selection.connect('changed', self._sig_changed)
 
 		view = rpc.session.rpc_exec_auth('/object', 'execute', 'ir.attachment', 'fields_view_get', False, 'tree')
 
@@ -74,7 +78,7 @@ class win_attach(object):
 
 		self.view.set_model(self.model)
 		self.view.show_all()
-		self.reload()
+		self.reload(preview=False)
 
 		dict = {
 			'on_attach_but_del_activate': self._sig_del,
@@ -101,7 +105,7 @@ class win_attach(object):
 		else:
 			common.warning('You must put a text comment to an attachement.','Text not saved !')
 
-	def _sig_del(self, *args):		
+	def _sig_del(self, *args):
 		model,iter = self.view.get_selection().get_selected()
 		if not iter:
 			return None
@@ -131,10 +135,13 @@ class win_attach(object):
 			return None
 		id = model.get_value(iter, 0)
 		if id:
-			data = rpc.session.rpc_exec_auth('/object', 'execute', 'ir.attachment', 'read', [id])
+			data = rpc.session.rpc_exec_auth('/object', 'execute', 'ir.attachment',
+					'read', [id])
 			if not len(data):
 				return None
-			filename = common.file_selection(_('Select the destination file'), filename=data[0]['datas_fname'], parent=self.win)
+			filename = common.file_selection(_('Save As...'),
+					filename=data[0]['datas_fname'], parent=self.win,
+					action=gtk.FILE_CHOOSER_ACTION_SAVE)
 			if not filename:
 				return None
 			try:
@@ -146,42 +153,34 @@ class win_attach(object):
 				common.message(_('Can not write file !'))
 
 	def _sig_add(self, *args):
-		dialog = gtk.FileChooserDialog("Open..",
-			self.win,
-			gtk.FILE_CHOOSER_ACTION_OPEN,
-			(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
-			gtk.STOCK_OPEN, gtk.RESPONSE_OK))
-		dialog.set_default_response(gtk.RESPONSE_OK)
-		dialog.set_select_multiple(True)
-		filter = gtk.FileFilter()
-		filter.set_name("All files")
-		filter.add_pattern("*")
-		dialog.add_filter(filter)
-		
-		filter = gtk.FileFilter()
-		filter.set_name("Images")
-		for mime in ("image/png","image/jpeg","image/gif"):
-			filter.add_mime_type(mime)
-		for pat in ("*.png","*.jpg","*.gif","*.tif","*.xpm"):
-			filter.add_pattern(pat)
-		dialog.add_filter(filter)
-		
-		response = dialog.run()
-		sfilenames = dialog.get_filenames()
-		dialog.destroy()
+		filter_all = gtk.FileFilter()
+		filter_all.set_name(_('All files'))
+		filter_all.add_pattern("*")
 
-		if response == gtk.RESPONSE_OK:
-			try:
-				for s in sfilenames:
-					value = file(s,'rb').read()
-					fname = os.path.basename(s)
-					id = rpc.session.rpc_exec_auth('/object', 'execute', 'ir.attachment', 'create', {'name':fname, 'datas':base64.encodestring(value), 'datas_fname':fname, 'res_model': self.ressource[0], 'res_id': self.ressource[1]})
-				self.reload(preview=False)
-				self.preview(id)
-			except:
-				pass
+		filter_image = gtk.FileFilter()
+		filter_image.set_name(_('Images'))
+		for mime in ("image/png", "image/jpeg", "image/gif"):
+			filter_image.add_mime_type(mime)
+		for pat in ("*.png", "*.jpg", "*.gif", "*.tif", "*.xpm"):
+			filter_image.add_pattern(pat)
 
-	def _sig_row_activated(self, *args):
+		filenames = common.file_selection(_('Open...'), preview=True, multi=True,
+				parent=self.win, filters=[filter_all, filter_image])
+		for filename in filenames:
+			value = file(filename, 'rb').read()
+			name = os.path.basename(filename)
+			id = rpc.session.rpc_exec_auth('/object', 'execute',
+					'ir.attachment', 'create', {
+						'name': name,
+						'datas': base64.encodestring(value),
+						'datas_fname': name,
+						'res_model': self.ressource[0],
+						'res_id': self.ressource[1]})
+			self.reload(preview=False)
+			self.preview(id)
+
+
+	def _sig_changed(self, *args):
 		model,iters = self.view.get_selection().get_selected_rows()
 		if not iters:
 			return None
@@ -206,9 +205,15 @@ class win_attach(object):
 		label = self.glade.get_widget('attach_title')
 		label.set_text(str(datas['name']))
 
-		decoder = {'jpg':'jpeg','jpeg':'jpeg','gif':'gif','png':'png'}
+		decoder = {'jpg': 'jpeg',
+				'jpeg': 'jpeg',
+				'gif': 'gif',
+				'png': 'png',
+				'bmp': 'bmp'}
 		ext = fname.split('.')[-1].lower()
-		if ext in ('jpg', 'jpeg', 'png', 'gif'):
+		img = self.glade.get_widget('attach_image')
+		img.clear()
+		if ext in ('jpg', 'jpeg', 'png', 'gif', 'bmp'):
 			try:
 				if not datas['link']:
 					dt = base64.decodestring(datas['datas'])
@@ -217,19 +222,19 @@ class win_attach(object):
 
 				def set_size(object, w, h):
 					allocation = self.win.get_allocation()
-					scale1 = 0.3*allocation.width / w
-					scale2 = 0.3*allocation.height / h
+					scale1 = 0.3 * float(allocation.width) / float(w)
+					scale2 = 0.3 * float(allocation.height) / float(h)
 					scale = min(scale1, scale2)
-					object.set_size(int(scale*w), int(scale*h))
+					if int(scale*w) > 0 and int(scale*h) > 0:
+						object.set_size(int(scale*w), int(scale*h))
 
-				loader = gtk.gdk.PixbufLoader (decoder[ext])
+				loader = gtk.gdk.PixbufLoader(decoder[ext])
 				loader.connect_after('size-prepared', set_size)
-				
-				loader.write (dt, len (dt))
-				pixbuf = loader.get_pixbuf ()
-				loader.close ()
 
-				img = self.glade.get_widget('attach_image')
+				loader.write(dt, len(dt))
+				pixbuf = loader.get_pixbuf()
+				loader.close()
+
 				img.set_from_pixbuf(pixbuf)
 			except Exception, e:
 				common.message(_('Unable to preview image file !\nVerify the format.'))
@@ -250,8 +255,6 @@ class win_attach(object):
 					os.startfile(fname)
 				else:
 					os.startfile(datas[0]['link'])
-			else:
-				self._sig_save(None)
 
 	def reload(self, preview=True):
 		self.model.clear()
@@ -263,9 +266,9 @@ class win_attach(object):
 			for x in range(len(self.fields_order)):
 				args.append(x+1)
 				if res['link']:
-					args.append('link : '+res[self.fields_order[x]])
+					args.append('link : '+str(res[self.fields_order[x]]))
 				else:
-					args.append(res[self.fields_order[x]])
+					args.append(str(res[self.fields_order[x]]))
 			self.model.set(num, 0, res['id'], *args)
 		if preview and len(res_ids):
 			self.preview(res_ids[0]['id'])
@@ -297,5 +300,6 @@ class win_attach(object):
 			else:
 				res = None
 				end = True
+		self.parent.present()
 		self.win.destroy()
 
