@@ -89,53 +89,22 @@ def open_excel(fields, result):
 def datas_read(ids, model, fields, fields_view, prefix=''):
 	datas = rpc.session.rpc_exec_auth('/object', 'execute', model, 'export_data', ids, fields)
 	return datas
-	rfield = []
-	todo = {}
-	for f in fields:
-		res = f.split('.')
-		if len(res)>1:
-			todo.setdefault(res[0], []).append('.'.join(res[1:]))
-			if not res[0] in rfield:
-				rfield.append(res[0])
-		else:
-			if not f in rfield:
-				rfield.append(f)
-
-	datas = rpc.session.rpc_exec_auth('/object', 'execute', model, 'read', ids, rfield)
-	for t in todo:
-		ids2 = []
-		for d in datas:
-			if fields_view[prefix+t]['type'] in ('one2many', 'many2many'):
-				ids2.extend(d[t])
-			elif fields_view[prefix+t]['type'] in ('many2one', 'one2one'):
-				if d[t]:
-					ids2.append(d[t][0])
-			else:
-				ids2.append(d[t])
-		datas2 = datas_read(ids2, fields_view[prefix+t]['relation'], todo[t], fields_view, prefix+t+'.')
-		res = {}
-		for d in datas2:
-			res[d['id']] = {}
-			for k in d:
-				if k!='id':
-					res[d['id']][t+'.'+k]=d[k]
-		for d in datas:
-			if fields_view[prefix+t]['type'] in ('one2many', 'many2many','many2one','one2one'):
-				d.update(res.get(d[t] and d[t][0],{}))
-			else:
-				d.update(res.get(d[t],{}))
-	return datas
 
 class win_export(object):
 	def __init__(self, model, ids, fields, preload = [], parent=None):
-		self.glade = glade.XML(common.terp_path("terp.glade"), 'win_save_as', gettext.textdomain())
+		self.glade = glade.XML(common.terp_path("terp.glade"), 'win_save_as',
+				gettext.textdomain())
 		self.win = self.glade.get_widget('win_save_as')
-		if parent:
-			self.win.set_transient_for(parent)
 		self.ids = ids
 		self.model = model
 		self.fields_data = {}
 
+		if parent is None:
+			parent = service.LocalService('gui.main').window
+		self.win.set_transient_for(parent)
+		self.win.set_icon(common.TINYERP_ICON)
+		self.parent = parent
+
 		self.view1 = gtk.TreeView()
 		self.view1.get_selection().set_mode(gtk.SELECTION_MULTIPLE)
 		self.glade.get_widget('exp_vp1').add(self.view1)
@@ -169,7 +138,7 @@ class win_export(object):
 					self.fields_data[prefix_node + field]['string'] = '%s%s' % (prefix_value, self.fields_data[prefix_node + field]['string'])
 				st_name = fields[field]['string'] or field 
 				node = self.model1.insert(prefix, 0, [st_name, prefix_node+field])
-				self.fields[prefix_node+field] = st_name
+				self.fields[prefix_node+field] = (st_name, fields[field].get('relation', False))
 				if fields[field].get('relation', False) and level>0:
 					fields2 = rpc.session.rpc_exec_auth('/object', 'execute', fields[field]['relation'], 'fields_get', False, rpc.session.context)
 					model_populate(fields2, prefix_node+field+'/', node, st_name+'/', level-1)
@@ -204,14 +173,18 @@ class win_export(object):
 
 	def sig_sel_all(self, widget=None):
 		self.model2.clear()
-		for field in self.fields.keys():
-			self.model2.set(self.model2.append(), 0, self.fields[field], 1, field)
+		for field, relation in self.fields.keys():
+			if not relation:
+				self.model2.set(self.model2.append(), 0, self.fields[field], 1, field)
 
 	def sig_sel(self, widget=None):
 		sel = self.view1.get_selection()
 		sel.selected_foreach(self._sig_sel_add)
 
 	def _sig_sel_add(self, store, path, iter):
+		name, relation = self.fields[store.get_value(iter,1)]
+		if relation:
+			return
 		num = self.model2.append()
 		self.model2.set(num, 0, store.get_value(iter,0), 1, store.get_value(iter,1))
 
@@ -263,6 +236,7 @@ class win_export(object):
 				fields2.append(self.model2.get_value(iter, 0))
 				iter = self.model2.iter_next(iter)
 			action = self.wid_action.get_active()
+			self.parent.present()
 			self.win.destroy()
 			result = datas_read(self.ids, self.model, fields, self.fields_data)
 			#result = []
@@ -272,11 +246,13 @@ class win_export(object):
 			if not action:
 				open_excel(fields2, result)
 			else:
-				fname = common.file_selection(_('Export Data'))
+				fname = common.file_selection(_('Save As...'),
+						parent=self.parent, action=gtk.FILE_CHOOSER_ACTION_SAVE)
 		 		if fname:
 					export_csv(fname, fields2, result, self.wid_write_field_names.get_active())
 			return True
 		else:
+			self.parent.present()
 			self.win.destroy()
 			return False
 
