@@ -35,9 +35,10 @@
 # Print or open a previewer
 #
 
-import os, base64, options
+import os, base64, options, sys
 import gc
 import common
+import time
 
 class Printer(object):
 
@@ -46,6 +47,7 @@ class Printer(object):
 			'pdf': self._findPDFOpener,
 			'html': self._findHTMLOpener,
 			'doc': self._findHTMLOpener,
+			'xls': self._findHTMLOpener,
 		}
 
 	def _findInPath(self, progs):
@@ -70,16 +72,29 @@ class Printer(object):
 		if os.name == 'nt':
 			return lambda fn: os.startfile(fn)
 		else:
-			if options.options['printer.preview']:
-				if options.options['printer.softpath_html'] == 'none':
-					prog = self._findInPath(['ooffice2', 'openoffice', 'firefox', 'mozilla', 'galeon'])
-					def opener(fn):
-						os.spawnv(os.P_NOWAIT, prog, (os.path.basename(prog),fn))
-					return opener
-				else:
-					def opener(fn):
-						os.system(options.options['printer.softpath_html'] + ' ' + fn)
-					return opener
+			if options.options['printer.softpath_html'] == 'none':
+				prog = self._findInPath(['ooffice', 'ooffice2', 'openoffice', 'firefox', 'mozilla', 'galeon'])
+				def opener(fn):
+					pid = os.fork()
+					if not pid:
+						pid = os.fork()
+						if not pid:
+							os.execv(prog, (os.path.basename(prog),fn))
+						time.sleep(0.1)
+						sys.exit(0)
+					os.waitpid(pid, 0)
+				return opener
+			else:
+				def opener(fn):
+					pid = os.fork()
+					if not pid:
+						pid = os.fork()
+						if not pid:
+							os.system(options.options['printer.softpath_html'] + ' ' + fn)
+						time.sleep(0.1)
+						sys.exit(0)
+					os.waitpid(pid, 0)
+				return opener
 
 	def _findPDFOpener(self):
 		if os.name == 'nt':
@@ -93,13 +108,27 @@ class Printer(object):
 		else:
 			if options.options['printer.preview']:
 				if options.options['printer.softpath'] == 'none':
-					prog = self._findInPath(['evince', 'xpdf', 'gpdf', 'kpdf', 'epdfview'])
+					prog = self._findInPath(['evince', 'xpdf', 'gpdf', 'kpdf', 'epdfview', 'acroread', 'open'])
 					def opener(fn):
-						os.spawnv(os.P_NOWAIT, prog, (os.path.basename(prog),fn))
+						pid = os.fork()
+						if not pid:
+							pid = os.fork()
+							if not pid:
+								os.execv(prog, (os.path.basename(prog), fn))
+							time.sleep(0.1)
+							sys.exit(0)
+						os.waitpid(pid, 0)
 					return opener
 				else:
 					def opener(fn):
-						os.spawnv(os.P_NOWAIT, options.options['printer.softpath'], (os.path.basename(options.options['printer.softpath']),fn))
+						pid = os.fork()
+						if not pid:
+							pid = os.fork()
+							if not pid:
+								os.execv(options.options['printer.softpath'], (os.path.basename(options.options['printer.softpath']),fn))
+							time.sleep(0.1)
+							sys.exit(0)
+						os.waitpid(pid, 0)
 					return opener
 			else:
 				return lambda fn: print_linux_filename(fn)
@@ -132,17 +161,19 @@ def print_data(data):
 		import tempfile
 		if data['format']=='html' and os.name=='nt':
 			data['format']='doc'
-		fp_name = tempfile.mktemp('.'+data['format'])
+		(fileno, fp_name) = tempfile.mkstemp('.'+data['format'], 'tinyerp_')
 		fp = file(fp_name, 'wb+')
 		fp.write(content)
 		fp.close()
+		os.close(fileno)
 		printer.print_file(fp_name, data['format'])
 	else:
 		fname = common.file_selection(_('Write Report to File'), data['format'])
-		try:
-			fp = file(filename,'wb+')
-			fp.write(content)
-			fp.close()
-		except:
-			common.message(_('Error writing the file!'))
+		if fname:
+			try:
+				fp = file(fname,'wb+')
+				fp.write(content)
+				fp.close()
+			except:
+				common.message(_('Error writing the file!'))
 
