@@ -2,7 +2,7 @@
 #
 # Copyright (c) 2004-2006 TINY SPRL. (http://tiny.be) All Rights Reserved.
 #
-# $Id: url.py 3861 2006-08-22 09:14:03Z pinky $
+# $Id$
 #
 # WARNING: This program as such is intended to be used by professional
 # programmers who take the whole responsability of assessing all potential
@@ -29,7 +29,6 @@
 
 import gettext
 import gtk
-from gtk import glade
 
 import common
 import interface
@@ -37,31 +36,40 @@ import tools
 
 class url(interface.widget_interface):
 	def __init__(self, window, parent, model, attrs={}):
-		self.win_gl = glade.XML(common.terp_path("terp.glade"), "widget_url", 
-								gettext.textdomain())
-		self.widget = self.win_gl.get_widget('widget_url')
+		interface.widget_interface.__init__(self, window, parent=parent, attrs=attrs)
 
-		self.entry = self.win_gl.get_widget('entry_url')
+		self.widget = gtk.HBox(homogeneous=False)
+
+		self.entry = gtk.Entry()
+		self.entry.set_max_length(int(attrs.get('size',16)))
+		self.entry.set_visibility(not attrs.get('invisible', False))
+		self.entry.set_width_chars(5)
+		self.entry.set_property('activates_default', True)
 		self.entry.connect('activate', self.sig_activate)
 		self.entry.connect('focus-in-event', lambda x,y: self._focus_in())
 		self.entry.connect('focus-out-event', lambda x,y: self._focus_out())
-		self.entry.set_max_length(int(attrs.get('size',16)))
-		self.entry.set_visibility(not attrs.get('invisible', False))
+		self.widget.pack_start(self.entry, expand=True, fill=True)
 
-		self.button = self.win_gl.get_widget('button_url')
+		self.button = gtk.Button()
+		img = gtk.Image()
+		img.set_from_stock('gtk-jump-to', gtk.ICON_SIZE_BUTTON)
+		self.button.set_image(img)
+		self.button.set_relief(gtk.RELIEF_NONE)
 		self.button.connect('clicked', self.button_clicked)
-		
-		interface.widget_interface.__init__(self, window, parent=parent, attrs=attrs)
+		self.button.set_alignment(0.5, 0.5)
+		self.button.set_property('can-focus', False)
+		self.widget.pack_start(self.button, expand=False, fill=False)
 
-	def set_value(self, model_field):
-		return model_field.set_client(self.entry.get_text() or False)
 
-	def display(self, model_field):
+	def set_value(self, model,model_field):
+		return model_field.set_client(model, self.entry.get_text() or False)
+
+	def display(self, model, model_field):
 		if not model_field:
 			self.entry.set_text('')
 			return False
-		super(url, self).display(model_field)
-		self.entry.set_text(model_field.get() or '')
+		super(url, self).display(model, model_field)
+		self.entry.set_text(model_field.get(model) or '')
 
 	def _readonly_set(self, value):
 		self.entry.set_editable(not value)
@@ -72,11 +80,26 @@ class url(interface.widget_interface):
 		if value:
 			tools.launch_browser(value)
 
+	def _color_widget(self):
+		return self.entry
+
 class email(url):
 	def button_clicked(self, widget):
 		value = self.entry.get_text()
 		if value:
 			tools.launch_browser('mailto:%s' % value)
 
+class callto(url):
+	def button_clicked(self, widget):
+		value = self.entry.get_text()
+		if value:
+			tools.launch_browser('callto:%s' % value)
+
+class sip(url):
+	def button_clicked(self, widget):
+		value = self.entry.get_text()
+		if value:
+			tools.launch_browser('sip:%s' % value)
+
 # vim:noexpandtab:
 
